@@ -52,27 +52,12 @@ use constant FILE_WILDCARDS => {
 };
 use constant MODEL_WILDCARD => join '|', @{&FILE_WILDCARDS}{qw(known stl obj amf prusa)};
 
+# Datadir provided on the command line.
 our $datadir;
 # If set, the "Controller" tab for the control of the printer over serial line and the serial port settings are hidden.
-our $no_controller;
 our $no_plater;
-our $autosave;
 our @cb;
 
-our $Settings = {
-    _ => {
-        version_check => 1,
-        autocenter => 1,
-        # Disable background processing by default as it is not stable.
-        background_processing => 0,
-        # If set, the "Controller" tab for the control of the printer over serial line and the serial port settings are hidden.
-        # By default, Prusa has the controller hidden.
-        no_controller => 1,
-        # If set, the "- default -" selections of print/filament/printer are suppressed, if there is a valid preset available.
-        no_defaults => 1,
-    },
-};
-
 our $small_font = Wx::SystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
 $small_font->SetPointSize(11) if &Wx::wxMAC;
 our $small_bold_font = Wx::SystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
@@ -82,134 +67,77 @@ our $medium_font = Wx::SystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
 $medium_font->SetPointSize(12);
 our $grey = Wx::Colour->new(200,200,200);
 
-#our $VERSION_CHECK_EVENT : shared = Wx::NewEventType;
-
-our $DLP_projection_screen;
-
 sub OnInit {
     my ($self) = @_;
     
-    $self->SetAppName('Slic3r');
+    $self->SetAppName('Slic3rPE');
     $self->SetAppDisplayName('Slic3r Prusa Edition');
     Slic3r::debugf "wxWidgets version %s, Wx version %s\n", &Wx::wxVERSION_STRING, $Wx::VERSION;
-    
-    $self->{notifier} = Slic3r::GUI::Notifier->new;
-    $self->{preset_bundle} = Slic3r::GUI::PresetBundle->new;
 
-    # locate or create data directory
+    # Set the Slic3r data directory at the Slic3r XS module.
     # Unix: ~/.Slic3r
     # Windows: "C:\Users\username\AppData\Roaming\Slic3r" or "C:\Documents and Settings\username\Application Data\Slic3r"
     # Mac: "~/Library/Application Support/Slic3r"
-    $datadir ||= Wx::StandardPaths::Get->GetUserDataDir;
-    my $enc_datadir = Slic3r::encode_path($datadir);
-    Slic3r::debugf "Data directory: %s\n", $datadir;
+    Slic3r::set_data_dir($datadir || Wx::StandardPaths::Get->GetUserDataDir);
+    Slic3r::GUI::set_wxapp($self);
     
-    # just checking for existence of $datadir is not enough: it may be an empty directory
+    $self->{notifier} = Slic3r::GUI::Notifier->new;
+    $self->{app_config} = Slic3r::GUI::AppConfig->new;
+    $self->{preset_bundle} = Slic3r::GUI::PresetBundle->new;
+    
+    # just checking for existence of Slic3r::data_dir is not enough: it may be an empty directory
     # supplied as argument to --datadir; in that case we should still run the wizard
-    my $run_wizard = (-d $enc_datadir && -e "$enc_datadir/slic3r.ini") ? 0 : 1;
-    foreach my $dir ($enc_datadir, "$enc_datadir/print", "$enc_datadir/filament", "$enc_datadir/printer") {
-        next if -d $dir;
-        if (!mkdir $dir) {
-            my $error = "Slic3r was unable to create its data directory at $dir ($!).";
-            warn "$error\n";
-            fatal_error(undef, $error);
-        }
+    eval { $self->{preset_bundle}->setup_directories() };
+    if ($@) {
+        warn $@ . "\n";
+        fatal_error(undef, $@);
     }
-    
+    my $run_wizard = ! $self->{app_config}->exists;
     # load settings
-    my $last_version;
-    if (-f "$enc_datadir/slic3r.ini") {
-        my $ini = eval { Slic3r::Config->read_ini("$datadir/slic3r.ini") };
-        $Settings = $ini if $ini;
-        $last_version = $Settings->{_}{version};
-        $Settings->{_}{autocenter} //= 1;
-        $Settings->{_}{background_processing} //= 1;
-        # If set, the "Controller" tab for the control of the printer over serial line and the serial port settings are hidden.
-        $Settings->{_}{no_controller} //= 1;
-        # If set, the "- default -" selections of print/filament/printer are suppressed, if there is a valid preset available.
-        $Settings->{_}{no_defaults} //= 1;
+    $self->{app_config}->load if ! $run_wizard;
+    $self->{app_config}->set('version', $Slic3r::VERSION);
+    $self->{app_config}->save;
+
+    # Suppress the '- default -' presets.
+    $self->{preset_bundle}->set_default_suppressed($self->{app_config}->get('no_defaults') ? 1 : 0);
+    eval { $self->{preset_bundle}->load_presets };
+    if ($@) {
+        warn $@ . "\n";
+        show_error(undef, $@);
     }
-    $Settings->{_}{version} = $Slic3r::VERSION;
-    $self->save_settings;
+    eval { $self->{preset_bundle}->load_selections($self->{app_config}) };
+    $run_wizard = 1 if $self->{preset_bundle}->has_defauls_only;
     
     # application frame
-    Wx::Image::AddHandler(Wx::PNGHandler->new);
+    Wx::Image::FindHandlerType(wxBITMAP_TYPE_PNG) || Wx::Image::AddHandler(Wx::PNGHandler->new);
     $self->{mainframe} = my $frame = Slic3r::GUI::MainFrame->new(
         # If set, the "Controller" tab for the control of the printer over serial line and the serial port settings are hidden.
-        no_controller   => $no_controller // $Settings->{_}{no_controller},
+        no_controller   => $self->{app_config}->get('no_controller'),
         no_plater       => $no_plater,
     );
     $self->SetTopWindow($frame);
-    
-    # load init bundle
-    {
-        my @dirs = ($FindBin::Bin);
-        if (&Wx::wxMAC) {
-            push @dirs, qw();
-        } elsif (&Wx::wxMSW) {
-            push @dirs, qw();
-        }
-        my $init_bundle = first { -e $_ } map "$_/.init_bundle.ini", @dirs;
-        if ($init_bundle) {
-            Slic3r::debugf "Loading config bundle from %s\n", $init_bundle;
-            $self->{mainframe}->load_configbundle($init_bundle, 1);
-            $run_wizard = 0;
-        }
-    }
-    
-    if (!$run_wizard && (!defined $last_version || $last_version ne $Slic3r::VERSION)) {
-        # user was running another Slic3r version on this computer
-        if (!defined $last_version || $last_version =~ /^0\./) {
-            show_info($self->{mainframe}, "Hello! Support material was improved since the "
-                . "last version of Slic3r you used. It is strongly recommended to revert "
-                . "your support material settings to the factory defaults and start from "
-                . "those. Enjoy and provide feedback!", "Support Material");
-        }
-        if (!defined $last_version || $last_version =~ /^(?:0|1\.[01])\./) {
-            show_info($self->{mainframe}, "Hello! In this version a new Bed Shape option was "
-                . "added. If the bed coordinates in the plater preview screen look wrong, go "
-                . "to Print Settings and click the \"Set\" button next to \"Bed Shape\".", "Bed Shape");
-        }
-    }
-    $self->{mainframe}->config_wizard if $run_wizard;
-    eval { $self->{preset_bundle}->load_presets($datadir) };
-    
-#    $self->check_version
-#        if $self->have_version_check
-#            && ($Settings->{_}{version_check} // 1)
-#            && (!$Settings->{_}{last_version_check} || (time - $Settings->{_}{last_version_check}) >= 86400);
-    
+
     EVT_IDLE($frame, sub {
         while (my $cb = shift @cb) {
             $cb->();
         }
+        $self->{app_config}->save if $self->{app_config}->dirty;
     });
-    
-#    EVT_COMMAND($self, -1, $VERSION_CHECK_EVENT, sub {
-#        my ($self, $event) = @_;
-#        my ($success, $response, $manual_check) = @{$event->GetData};
-#        
-#        if ($success) {
-#            if ($response =~ /^obsolete ?= ?([a-z0-9.-]+,)*\Q$Slic3r::VERSION\E(?:,|$)/) {
-#                my $res = Wx::MessageDialog->new(undef, "A new version is available. Do you want to open the Slic3r website now?",
-#                    'Update', wxYES_NO | wxCANCEL | wxYES_DEFAULT | wxICON_INFORMATION | wxICON_ERROR)->ShowModal;
-#                Wx::LaunchDefaultBrowser('http://slic3r.org/') if $res == wxID_YES;
-#            } else {
-#                Slic3r::GUI::show_info(undef, "You're using the latest version. No updates are available.") if $manual_check;
-#            }
-#            $Settings->{_}{last_version_check} = time();
-#            $self->save_settings;
-#        } else {
-#            Slic3r::GUI::show_error(undef, "Failed to check for updates. Try later.") if $manual_check;
-#        }
-#    });
+
+    if ($run_wizard) {
+        # On OSX the UI was not initialized correctly if the wizard was called
+        # before the UI was up and running.
+        $self->CallAfter(sub {
+            # Run the config wizard, don't offer the "reset user profile" checkbox.
+            $self->{mainframe}->config_wizard(1);
+        });
+    }
     
     return 1;
 }
 
 sub about {
     my ($self) = @_;
-    
     my $about = Slic3r::GUI::AboutDialog->new(undef);
     $about->ShowModal;
     $about->Destroy;
@@ -217,7 +145,6 @@ sub about {
 
 sub system_info {
     my ($self) = @_;
-
     my $slic3r_info = Slic3r::slic3r_info(format => 'html');
     my $copyright_info = Slic3r::copyright_info(format => 'html');
     my $system_info = Slic3r::system_info(format => 'html');
@@ -295,11 +222,6 @@ sub notify {
     $self->{notifier}->notify($message);
 }
 
-sub save_settings {
-    my ($self) = @_;
-    Slic3r::Config->write_ini("$datadir/slic3r.ini", $Settings);
-}
-
 # Called after the Preferences dialog is closed and the program settings are saved.
 # Update the UI based on the current preferences.
 sub update_ui_from_settings {
@@ -307,64 +229,11 @@ sub update_ui_from_settings {
     $self->{mainframe}->update_ui_from_settings;
 }
 
-sub presets {
-    my ($self, $section) = @_;
-    
-    my %presets = ();
-    opendir my $dh, Slic3r::encode_path("$Slic3r::GUI::datadir/$section")
-        or die "Failed to read directory $Slic3r::GUI::datadir/$section (errno: $!)\n";
-    # Instead of using the /i modifier for case-insensitive matching, the case insensitivity is expressed
-    # explicitely to avoid having to bundle the UTF8 Perl library.
-    foreach my $file (grep /\.[iI][nN][iI]$/, readdir $dh) {
-        $file = Slic3r::decode_path($file);
-        my $name = basename($file);
-        $name =~ s/\.ini$//;
-        $presets{$name} = "$Slic3r::GUI::datadir/$section/$file";
-    }
-    closedir $dh;
-    
-    return %presets;
-}
-
-#sub have_version_check {
-#    my ($self) = @_;
-#    
-#    # return an explicit 0
-#    return ($Slic3r::have_threads && $Slic3r::build && $have_LWP) || 0;
-#}
-
-#sub check_version {
-#    my ($self, $manual_check) = @_;
-#    
-#    Slic3r::debugf "Checking for updates...\n";
-#    
-#    @_ = ();
-#    threads->create(sub {
-#        my $ua = LWP::UserAgent->new;
-#        $ua->timeout(10);
-#        my $response = $ua->get('http://slic3r.org/updatecheck');
-#        Wx::PostEvent($self, Wx::PlThreadEvent->new(-1, $VERSION_CHECK_EVENT,
-#            threads::shared::shared_clone([ $response->is_success, $response->decoded_content, $manual_check ])));
-#        Slic3r::thread_cleanup();
-#    })->detach;
-#}
-
-sub output_path {
-    my ($self, $dir) = @_;
-    
-    return ($Settings->{_}{last_output_path} && $Settings->{_}{remember_output_path})
-        ? $Settings->{_}{last_output_path}
-        : $dir;
-}
-
 sub open_model {
     my ($self, $window) = @_;
     
-    my $dir = $Slic3r::GUI::Settings->{recent}{skein_directory}
-           || $Slic3r::GUI::Settings->{recent}{config_directory}
-           || '';
-    
-    my $dialog = Wx::FileDialog->new($window // $self->GetTopWindow, 'Choose one or more files (STL/OBJ/AMF/PRUSA):', $dir, "",
+    my $dialog = Wx::FileDialog->new($window // $self->GetTopWindow, 'Choose one or more files (STL/OBJ/AMF/PRUSA):', 
+        $self->{app_config}->get_last_dir, "",
         MODEL_WILDCARD, wxFD_OPEN | wxFD_MULTIPLE | wxFD_FILE_MUST_EXIST);
     if ($dialog->ShowModal != wxID_OK) {
         $dialog->Destroy;
@@ -380,31 +249,6 @@ sub CallAfter {
     push @cb, $cb;
 }
 
-sub scan_serial_ports {
-    my ($self) = @_;
-    
-    my @ports = ();
-    
-    if ($^O eq 'MSWin32') {
-        # Windows
-        if (eval "use Win32::TieRegistry; 1") {
-            my $ts = Win32::TieRegistry->new("HKEY_LOCAL_MACHINE\\HARDWARE\\DEVICEMAP\\SERIALCOMM",
-                { Access => 'KEY_READ' });
-            if ($ts) {
-                # when no serial ports are available, the registry key doesn't exist and 
-                # TieRegistry->new returns undef
-                $ts->Tie(\my %reg);
-                push @ports, sort values %reg;
-            }
-        }
-    } else {
-        # UNIX and OS X
-        push @ports, glob '/dev/{ttyUSB,ttyACM,tty.,cu.,rfcomm}*';
-    }
-    
-    return grep !/Bluetooth|FireFly/, @ports;
-}
-
 sub append_menu_item {
     my ($self, $menu, $string, $description, $cb, $id, $icon, $kind) = @_;
     
@@ -434,32 +278,31 @@ sub set_menu_item_icon {
     
     # SetBitmap was not available on OS X before Wx 0.9927
     if ($icon && $menuItem->can('SetBitmap')) {
-        $menuItem->SetBitmap(Wx::Bitmap->new($Slic3r::var->($icon), wxBITMAP_TYPE_PNG));
+        $menuItem->SetBitmap(Wx::Bitmap->new(Slic3r::var($icon), wxBITMAP_TYPE_PNG));
     }
 }
 
 sub save_window_pos {
     my ($self, $window, $name) = @_;
     
-    $Settings->{_}{"${name}_pos"}  = join ',', $window->GetScreenPositionXY;
-    $Settings->{_}{"${name}_size"} = join ',', $window->GetSizeWH;
-    $Settings->{_}{"${name}_maximized"}      = $window->IsMaximized;
-    $self->save_settings;
+    $self->{app_config}->set("${name}_pos", join ',', $window->GetScreenPositionXY);
+    $self->{app_config}->set("${name}_size", join ',', $window->GetSizeWH);
+    $self->{app_config}->set("${name}_maximized", $window->IsMaximized);
+    $self->{app_config}->save;
 }
 
 sub restore_window_pos {
     my ($self, $window, $name) = @_;
-    
-    if (defined $Settings->{_}{"${name}_pos"}) {
-        my $size = [ split ',', $Settings->{_}{"${name}_size"}, 2 ];
+    if ($self->{app_config}->has("${name}_pos")) {
+        my $size = [ split ',', $self->{app_config}->get("${name}_size"), 2 ];
         $window->SetSize($size);
         
         my $display = Wx::Display->new->GetClientArea();
-        my $pos = [ split ',', $Settings->{_}{"${name}_pos"}, 2 ];
+        my $pos = [ split ',', $self->{app_config}->get("${name}_pos"), 2 ];
         if (($pos->[0] + $size->[0]/2) < $display->GetRight && ($pos->[1] + $size->[1]/2) < $display->GetBottom) {
             $window->Move($pos);
         }
-        $window->Maximize(1) if $Settings->{_}{"${name}_maximized"};
+        $window->Maximize(1) if $self->{app_config}->get("${name}_maximized");
     }
 }
 
