@@ -15,7 +15,7 @@ package Slic3r::GUI::3DScene::Base;
 use strict;
 use warnings;
 
-use Wx qw(:timer :bitmap :icon :dialog);
+use Wx qw(wxTheApp :timer :bitmap :icon :dialog);
 use Wx::Event qw(EVT_PAINT EVT_SIZE EVT_ERASE_BACKGROUND EVT_IDLE EVT_MOUSEWHEEL EVT_MOUSE_EVENTS EVT_CHAR EVT_TIMER);
 # must load OpenGL *before* Wx::GLCanvas
 use OpenGL qw(:glconstants :glfunctions :glufunctions :gluconstants);
@@ -108,6 +108,7 @@ sub new {
     # We can only enable multi sample anti aliasing wih wxWidgets 3.0.3 and with a hacked Wx::GLCanvas,
     # which exports some new WX_GL_XXX constants, namely WX_GL_SAMPLE_BUFFERS and WX_GL_SAMPLES.
     my $can_multisample =
+        ! wxTheApp->{app_config}->get('use_legacy_opengl') &&
         Wx::wxVERSION >= 3.000003 &&
         defined Wx::GLCanvas->can('WX_GL_SAMPLE_BUFFERS') &&
         defined Wx::GLCanvas->can('WX_GL_SAMPLES');
@@ -956,6 +957,16 @@ sub UseVBOs {
     my ($self) = @_;
 
     if (! defined ($self->{use_VBOs})) {
+        my $use_legacy = wxTheApp->{app_config}->get('use_legacy_opengl');
+        if ($use_legacy eq '1') {
+            # Disable OpenGL 2.0 rendering.
+            $self->{use_VBOs} = 0;
+            # Don't enable the layer editing tool.
+            $self->{layer_editing_enabled} = 0;
+            # 2 means failed
+            $self->{layer_editing_initialized} = 2;
+            return 0;
+        }
         # This is a special path for wxWidgets on GTK, where an OpenGL context is initialized
         # first when an OpenGL widget is shown for the first time. How ugly.
         return 0 if (! $self->init && $^O eq 'linux');
@@ -1146,18 +1157,17 @@ sub Render {
     glLightfv_p(GL_LIGHT1, GL_POSITION, 1, 0, 1, 0);
     
     if ($self->enable_picking) {
-        # Render the object for picking.
-        # FIXME This cannot possibly work in a multi-sampled context as the color gets mangled by the anti-aliasing.
-        # Better to use software ray-casting on a bounding-box hierarchy.
-        glPushAttrib(GL_ENABLE_BIT);
-        glDisable(GL_MULTISAMPLE) if ($self->{can_multisample});
-        glDisable(GL_LIGHTING);
-        glDisable(GL_BLEND);
-        $self->draw_volumes(1);
-        glFlush();
-        glFinish();
-        
         if (my $pos = $self->_mouse_pos) {
+            # Render the object for picking.
+            # FIXME This cannot possibly work in a multi-sampled context as the color gets mangled by the anti-aliasing.
+            # Better to use software ray-casting on a bounding-box hierarchy.
+            glPushAttrib(GL_ENABLE_BIT);
+            glDisable(GL_MULTISAMPLE) if ($self->{can_multisample});
+            glDisable(GL_LIGHTING);
+            glDisable(GL_BLEND);
+            $self->draw_volumes(1);
+            glPopAttrib();
+            glFlush();
             my $col = [ glReadPixels_p($pos->x, $self->GetSize->GetHeight - $pos->y, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE) ];
             my $volume_idx = $col->[0] + $col->[1]*256 + $col->[2]*256*256;
             $self->_hover_volume_idx(undef);
@@ -1173,11 +1183,8 @@ sub Render {
                 
                 $self->on_hover->($volume_idx) if $self->on_hover;
             }
+            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
         }
-        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-        glFlush();
-        glFinish();
-        glPopAttrib();
     }
     
     # draw fixed background
@@ -1308,9 +1315,6 @@ sub Render {
     $self->draw_active_object_annotations;
     
     $self->SwapBuffers();
-
-    # Calling glFinish has a performance penalty, but it seems to fix some OpenGL driver hang-up with extremely large scenes.
-#    glFinish();
 }
 
 sub draw_volumes {
@@ -1390,7 +1394,7 @@ sub _load_image_set_texture {
     my ($self, $file_name) = @_;
     # Load a PNG with an alpha channel.
     my $img = Wx::Image->new;
-    $img->LoadFile($Slic3r::var->($file_name), wxBITMAP_TYPE_PNG);
+    $img->LoadFile(Slic3r::var($file_name), wxBITMAP_TYPE_PNG);
     # Get RGB & alpha raw data from wxImage, interleave them into a Perl array.
     my @rgb = unpack 'C*', $img->GetData();
     my @alpha = $img->HasAlpha ? unpack 'C*', $img->GetAlpha() : (255) x (int(@rgb) / 3);
