@@ -15,7 +15,7 @@ typedef Pointf3 Sizef3;
 template <class PointClass>
 class BoundingBoxBase
 {
-    public:
+public:
     PointClass min;
     PointClass max;
     bool defined;
@@ -30,7 +30,7 @@ class BoundingBoxBase
     void scale(double factor);
     PointClass size() const;
     double radius() const;
-    void translate(coordf_t x, coordf_t y) { this->min.translate(x, y); this->max.translate(x, y); }
+    void translate(coordf_t x, coordf_t y) { assert(this->defined); this->min.translate(x, y); this->max.translate(x, y); }
     void translate(const Pointf &pos) { this->translate(pos.x, pos.y); }
     void offset(coordf_t delta);
     PointClass center() const;
@@ -42,12 +42,14 @@ class BoundingBoxBase
         return ! (this->max.x < other.min.x || this->min.x > other.max.x ||
                   this->max.y < other.min.y || this->min.y > other.max.y);
     }
+    bool operator==(const BoundingBoxBase<PointClass> &rhs) { return this->min == rhs.min && this->max == rhs.max; }
+    bool operator!=(const BoundingBoxBase<PointClass> &rhs) { return ! (*this == rhs); }
 };
 
 template <class PointClass>
 class BoundingBox3Base : public BoundingBoxBase<PointClass>
 {
-    public:
+public:
     BoundingBox3Base() : BoundingBoxBase<PointClass>() {};
     BoundingBox3Base(const PointClass &pmin, const PointClass &pmax) : 
         BoundingBoxBase<PointClass>(pmin, pmax) 
@@ -66,7 +68,7 @@ class BoundingBox3Base : public BoundingBoxBase<PointClass>
 
 class BoundingBox : public BoundingBoxBase<Point>
 {
-    public:
+public:
     void polygon(Polygon* polygon) const;
     Polygon polygon() const;
     BoundingBox rotated(double angle) const;
@@ -85,36 +87,30 @@ class BoundingBox : public BoundingBoxBase<Point>
     friend BoundingBox get_extents_rotated(const Points &points, double angle);
 };
 
-/*
-class BoundingBox3  : public BoundingBox3Base<Point3> {};
-*/
+class BoundingBox3  : public BoundingBox3Base<Point3> 
+{
+public:
+    BoundingBox3() : BoundingBox3Base<Point3>() {};
+    BoundingBox3(const Point3 &pmin, const Point3 &pmax) : BoundingBox3Base<Point3>(pmin, pmax) {};
+    BoundingBox3(const std::vector<Point3> &points) : BoundingBox3Base<Point3>(points) {};
+};
 
-class BoundingBoxf : public BoundingBoxBase<Pointf> {
-    public:
+class BoundingBoxf : public BoundingBoxBase<Pointf> 
+{
+public:
     BoundingBoxf() : BoundingBoxBase<Pointf>() {};
     BoundingBoxf(const Pointf &pmin, const Pointf &pmax) : BoundingBoxBase<Pointf>(pmin, pmax) {};
     BoundingBoxf(const std::vector<Pointf> &points) : BoundingBoxBase<Pointf>(points) {};
 };
 
-class BoundingBoxf3 : public BoundingBox3Base<Pointf3> {
-    public:
+class BoundingBoxf3 : public BoundingBox3Base<Pointf3> 
+{
+public:
     BoundingBoxf3() : BoundingBox3Base<Pointf3>() {};
     BoundingBoxf3(const Pointf3 &pmin, const Pointf3 &pmax) : BoundingBox3Base<Pointf3>(pmin, pmax) {};
     BoundingBoxf3(const std::vector<Pointf3> &points) : BoundingBox3Base<Pointf3>(points) {};
 };
 
-template<typename VT>
-inline bool operator==(const BoundingBoxBase<VT> &bb1, const BoundingBoxBase<VT> &bb2)
-{
-    return bb1.min == bb2.min && bb1.max == bb2.max;
-}
-
-template<typename VT>
-inline bool operator!=(const BoundingBoxBase<VT> &bb1, const BoundingBoxBase<VT> &bb2)
-{
-    return !(bb1 == bb2);
-}
-
 template<typename VT>
 inline bool empty(const BoundingBoxBase<VT> &bb)
 {
