@@ -1,6 +1,6 @@
 #include "Config.hpp"
+#include "Utils.hpp"
 #include <assert.h>
-#include <ctime>
 #include <fstream>
 #include <iostream>
 #include <exception> // std::runtime_error
@@ -16,13 +16,8 @@
 #include <boost/nowide/cenv.hpp>
 #include <boost/nowide/fstream.hpp>
 #include <boost/property_tree/ini_parser.hpp>
-#include <boost/property_tree/ptree.hpp>
 #include <string.h>
 
-#if defined(_WIN32) && !defined(setenv) && defined(_putenv_s)
-#define setenv(k, v, o) _putenv_s(k, v)
-#endif
-
 namespace Slic3r {
 
 std::string escape_string_cstyle(const std::string &str)
@@ -235,13 +230,16 @@ bool ConfigBase::set_deserialize_raw(const t_config_option_key &opt_key_src, con
 {
     t_config_option_key opt_key = opt_key_src;
     // Try to deserialize the option by its name.
-    const ConfigOptionDef* optdef = this->def()->get(opt_key);
+    const ConfigDef       *def    = this->def();
+    if (def == nullptr)
+        throw NoDefinitionException();
+    const ConfigOptionDef *optdef = def->get(opt_key);
     if (optdef == nullptr) {
         // If we didn't find an option, look for any other option having this as an alias.
-        for (const auto &opt : this->def()->options) {
+        for (const auto &opt : def->options) {
             for (const t_config_option_key &opt_key2 : opt.second.aliases) {
                 if (opt_key2 == opt_key) {
-                    opt_key = opt_key2;
+                    opt_key = opt.first;
                     optdef = &opt.second;
                     break;
                 }
@@ -278,10 +276,16 @@ double ConfigBase::get_abs_value(const t_config_option_key &opt_key) const
         return static_cast<const ConfigOptionFloat*>(raw_opt)->value;
     if (raw_opt->type() == coFloatOrPercent) {
         // Get option definition.
-        const ConfigOptionDef *def = this->def()->get(opt_key);
-        assert(def != nullptr);
+        const ConfigDef *def = this->def();
+        if (def == nullptr)
+            throw NoDefinitionException();
+        const ConfigOptionDef *opt_def = def->get(opt_key);
+        assert(opt_def != nullptr);
         // Compute absolute value over the absolute value of the base option.
-        return static_cast<const ConfigOptionFloatOrPercent*>(raw_opt)->get_abs_value(this->get_abs_value(def->ratio_over));
+        //FIXME there are some ratio_over chains, which end with empty ratio_with.
+        // For example, XXX_extrusion_width parameters are not handled by get_abs_value correctly.
+        return opt_def->ratio_over.empty() ? 0. : 
+            static_cast<const ConfigOptionFloatOrPercent*>(raw_opt)->get_abs_value(this->get_abs_value(opt_def->ratio_over));
     }
     throw std::runtime_error("ConfigBase::get_abs_value(): Not a valid option type for get_abs_value()");
 }
@@ -301,7 +305,6 @@ double ConfigBase::get_abs_value(const t_config_option_key &opt_key, double rati
 
 void ConfigBase::setenv_()
 {
-#ifdef setenv
     t_config_option_keys opt_keys = this->keys();
     for (t_config_option_keys::const_iterator it = opt_keys.begin(); it != opt_keys.end(); ++it) {
         // prepend the SLIC3R_ prefix
@@ -314,18 +317,29 @@ void ConfigBase::setenv_()
         for (size_t i = 0; i < envname.size(); ++i)
             envname[i] = (envname[i] <= 'z' && envname[i] >= 'a') ? envname[i]-('a'-'A') : envname[i];
         
-        setenv(envname.c_str(), this->serialize(*it).c_str(), 1);
+        boost::nowide::setenv(envname.c_str(), this->serialize(*it).c_str(), 1);
     }
-#endif
 }
 
 void ConfigBase::load(const std::string &file)
 {
-    namespace pt = boost::property_tree;
-    pt::ptree tree;
+    if (boost::iends_with(file, ".gcode") || boost::iends_with(file, ".g"))
+        this->load_from_gcode(file);
+    else
+        this->load_from_ini(file);
+}
+
+void ConfigBase::load_from_ini(const std::string &file)
+{
+    boost::property_tree::ptree tree;
     boost::nowide::ifstream ifs(file);
-    pt::read_ini(ifs, tree);
-    for (const pt::ptree::value_type &v : tree) {
+    boost::property_tree::read_ini(ifs, tree);
+    this->load(tree);
+}
+
+void ConfigBase::load(const boost::property_tree::ptree &tree)
+{
+    for (const boost::property_tree::ptree::value_type &v : tree) {
         try {
             t_config_option_key opt_key = v.first;
             this->set_deserialize(opt_key, v.second.get_value<std::string>());
@@ -414,18 +428,24 @@ void ConfigBase::save(const std::string &file) const
 {
     boost::nowide::ofstream c;
     c.open(file, std::ios::out | std::ios::trunc);
-    {
-        std::time_t now;
-        time(&now);
-        char buf[sizeof "0000-00-00 00:00:00"];
-        strftime(buf, sizeof(buf), "%F %T", gmtime(&now));
-        c << "# generated by Slic3r " << SLIC3R_VERSION << " on " << buf << std::endl;
-    }
+    c << "# " << Slic3r::header_slic3r_generated() << std::endl;
     for (const std::string &opt_key : this->keys())
         c << opt_key << " = " << this->serialize(opt_key) << std::endl;
     c.close();
 }
 
+bool DynamicConfig::operator==(const DynamicConfig &rhs) const
+{
+    t_options_map::const_iterator it1     = this->options.begin();
+    t_options_map::const_iterator it1_end = this->options.end();
+    t_options_map::const_iterator it2     = rhs.options.begin();
+    t_options_map::const_iterator it2_end = rhs.options.end();
+    for (; it1 != it1_end && it2 != it2_end; ++ it1, ++ it2)
+        if (*it1->second != *it2->second)
+            return false;
+    return it1 == it1_end && it2 == it2_end;
+}
+
 ConfigOption* DynamicConfig::optptr(const t_config_option_key &opt_key, bool create)
 {
     t_options_map::iterator it = options.find(opt_key);
@@ -436,7 +456,10 @@ ConfigOption* DynamicConfig::optptr(const t_config_option_key &opt_key, bool cre
         // Option was not found and a new option shall not be created.
         return nullptr;
     // Try to create a new ConfigOption.
-    const ConfigOptionDef *optdef = this->def()->get(opt_key);
+    const ConfigDef       *def    = this->def();
+    if (def == nullptr)
+        throw NoDefinitionException();
+    const ConfigOptionDef *optdef = def->get(opt_key);
     if (optdef == nullptr)
 //        throw std::runtime_error(std::string("Invalid option name: ") + opt_key);
         // Let the parent decide what to do if the opt_key is not defined by this->def().
