@@ -3,6 +3,8 @@
 #include <string>
 #include <expat/expat.h>
 
+#include <boost/nowide/cstdio.hpp>
+
 #include "../libslic3r.h"
 #include "../Model.hpp"
 #include "AMF.hpp"
@@ -480,7 +482,7 @@ bool load_amf(const char *path, Model *model)
         return false;
     }
 
-    FILE *pFile = ::fopen(path, "rt");
+    FILE *pFile = boost::nowide::fopen(path, "rt");
     if (pFile == nullptr) {
         printf("Cannot open file %s\n", path);
         return false;
@@ -522,7 +524,7 @@ bool load_amf(const char *path, Model *model)
 
 bool store_amf(const char *path, Model *model)
 {
-    FILE *file = ::fopen(path, "wb");
+    FILE *file = boost::nowide::fopen(path, "wb");
     if (file == nullptr)
         return false;
 
