@@ -26,7 +26,8 @@ void PressureEqualizer::reset()
     circular_buffer_items   = 0;
     circular_buffer.assign(circular_buffer_size, GCodeLine());
 
-    output_buffer.clear();
+    // Preallocate some data, so that output_buffer.data() will return an empty string.
+    output_buffer.assign(32, 0);
     output_buffer_length    = 0;
 
     m_current_extruder = 0;
