@@ -1,6 +1,7 @@
 #include "Layer.hpp"
 #include "BridgeDetector.hpp"
 #include "ClipperUtils.hpp"
+#include "Geometry.hpp"
 #include "PerimeterGenerator.hpp"
 #include "Print.hpp"
 #include "Surface.hpp"
@@ -87,8 +88,7 @@ LayerRegion::make_perimeters(const SurfaceCollection &slices, SurfaceCollection*
 //#define EXTERNAL_SURFACES_OFFSET_PARAMETERS ClipperLib::jtMiter, 1.5
 #define EXTERNAL_SURFACES_OFFSET_PARAMETERS ClipperLib::jtSquare, 0.
 
-void
-LayerRegion::process_external_surfaces(const Layer* lower_layer)
+void LayerRegion::process_external_surfaces(const Layer* lower_layer)
 {
     const Surfaces &surfaces = this->fill_surfaces.surfaces;
     const double margin = scale_(EXTERNAL_INFILL_MARGIN);
@@ -260,7 +260,7 @@ LayerRegion::process_external_surfaces(const Layer* lower_layer)
                 #ifdef SLIC3R_DEBUG
                 printf("Processing bridge at layer " PRINTF_ZU ":\n", this->layer()->id());
                 #endif
-                if (bd.detect_angle()) {
+                if (bd.detect_angle(Geometry::deg2rad(this->region()->config.bridge_angle.value))) {
                     bridges[idx_last].bridge_angle = bd.angle;
                     if (this->layer()->object()->config.support_material) {
                         polygons_append(this->bridged, bd.coverage());
@@ -389,8 +389,7 @@ LayerRegion::infill_area_threshold() const
     return ss*ss;
 }
 
-
-void LayerRegion::export_region_slices_to_svg(const char *path)
+void LayerRegion::export_region_slices_to_svg(const char *path) const
 {
     BoundingBox bbox;
     for (Surfaces::const_iterator surface = this->slices.surfaces.begin(); surface != this->slices.surfaces.end(); ++surface)
@@ -410,14 +409,14 @@ void LayerRegion::export_region_slices_to_svg(const char *path)
 }
 
 // Export to "out/LayerRegion-name-%d.svg" with an increasing index with every export.
-void LayerRegion::export_region_slices_to_svg_debug(const char *name)
+void LayerRegion::export_region_slices_to_svg_debug(const char *name) const
 {
     static std::map<std::string, size_t> idx_map;
     size_t &idx = idx_map[name];
     this->export_region_slices_to_svg(debug_out_path("LayerRegion-slices-%s-%d.svg", name, idx ++).c_str());
 }
 
-void LayerRegion::export_region_fill_surfaces_to_svg(const char *path) 
+void LayerRegion::export_region_fill_surfaces_to_svg(const char *path) const
 {
     BoundingBox bbox;
     for (Surfaces::const_iterator surface = this->fill_surfaces.surfaces.begin(); surface != this->fill_surfaces.surfaces.end(); ++surface)
@@ -428,16 +427,16 @@ void LayerRegion::export_region_fill_surfaces_to_svg(const char *path)
 
     SVG svg(path, bbox);
     const float transparency = 0.5f;
-    for (Surfaces::const_iterator surface = this->fill_surfaces.surfaces.begin(); surface != this->fill_surfaces.surfaces.end(); ++surface) {
-        svg.draw(surface->expolygon, surface_type_to_color_name(surface->surface_type), transparency);
-        svg.draw_outline(surface->expolygon, "black", "blue", scale_(0.05)); 
+    for (const Surface &surface : this->fill_surfaces.surfaces) {
+        svg.draw(surface.expolygon, surface_type_to_color_name(surface.surface_type), transparency);
+        svg.draw_outline(surface.expolygon, "black", "blue", scale_(0.05)); 
     }
     export_surface_type_legend_to_svg(svg, legend_pos);
     svg.Close();
 }
 
 // Export to "out/LayerRegion-name-%d.svg" with an increasing index with every export.
-void LayerRegion::export_region_fill_surfaces_to_svg_debug(const char *name)
+void LayerRegion::export_region_fill_surfaces_to_svg_debug(const char *name) const
 {
     static std::map<std::string, size_t> idx_map;
     size_t &idx = idx_map[name];
