@@ -81,80 +81,80 @@ bool Print::invalidate_state_by_config_options(const std::vector<t_config_option
 
     // Cache the plenty of parameters, which influence the G-code generator only,
     // or they are only notes not influencing the generated G-code.
-    static std::unordered_set<std::string> steps_ignore;
-    if (steps_ignore.empty()) {
-        steps_ignore.insert("avoid_crossing_perimeters");
-        steps_ignore.insert("bed_shape");
-        steps_ignore.insert("bed_temperature");
-        steps_ignore.insert("before_layer_gcode");
-        steps_ignore.insert("bridge_acceleration");
-        steps_ignore.insert("bridge_fan_speed");
-        steps_ignore.insert("cooling");
-        steps_ignore.insert("default_acceleration");
-        steps_ignore.insert("deretract_speed");
-        steps_ignore.insert("disable_fan_first_layers");
-        steps_ignore.insert("duplicate_distance");
-        steps_ignore.insert("end_gcode");
-        steps_ignore.insert("end_filament_gcode");
-        steps_ignore.insert("extrusion_axis");
-        steps_ignore.insert("extruder_clearance_height");
-        steps_ignore.insert("extruder_clearance_radius");
-        steps_ignore.insert("extruder_colour");
-        steps_ignore.insert("extruder_offset");
-        steps_ignore.insert("extrusion_multiplier");
-        steps_ignore.insert("fan_always_on");
-        steps_ignore.insert("fan_below_layer_time");
-        steps_ignore.insert("filament_colour");
-        steps_ignore.insert("filament_diameter");
-        steps_ignore.insert("filament_density");
-        steps_ignore.insert("filament_notes");
-        steps_ignore.insert("filament_cost");
-        steps_ignore.insert("filament_max_volumetric_speed");
-        steps_ignore.insert("first_layer_acceleration");
-        steps_ignore.insert("first_layer_bed_temperature");
-        steps_ignore.insert("first_layer_speed");
-        steps_ignore.insert("gcode_comments");
-        steps_ignore.insert("gcode_flavor");
-        steps_ignore.insert("infill_acceleration");
-        steps_ignore.insert("infill_first");
-        steps_ignore.insert("layer_gcode");
-        steps_ignore.insert("min_fan_speed");
-        steps_ignore.insert("max_fan_speed");
-        steps_ignore.insert("min_print_speed");
-        steps_ignore.insert("max_print_speed");
-        steps_ignore.insert("max_volumetric_speed");
-        steps_ignore.insert("max_volumetric_extrusion_rate_slope_positive");
-        steps_ignore.insert("max_volumetric_extrusion_rate_slope_negative");
-        steps_ignore.insert("notes");
-        steps_ignore.insert("only_retract_when_crossing_perimeters");
-        steps_ignore.insert("output_filename_format");
-        steps_ignore.insert("perimeter_acceleration");
-        steps_ignore.insert("post_process");
-        steps_ignore.insert("printer_notes");
-        steps_ignore.insert("retract_before_travel");
-        steps_ignore.insert("retract_before_wipe");
-        steps_ignore.insert("retract_layer_change");
-        steps_ignore.insert("retract_length");
-        steps_ignore.insert("retract_length_toolchange");
-        steps_ignore.insert("retract_lift");
-        steps_ignore.insert("retract_lift_above");
-        steps_ignore.insert("retract_lift_below");
-        steps_ignore.insert("retract_restart_extra");
-        steps_ignore.insert("retract_restart_extra_toolchange");
-        steps_ignore.insert("retract_speed");
-        steps_ignore.insert("slowdown_below_layer_time");
-        steps_ignore.insert("standby_temperature_delta");
-        steps_ignore.insert("start_gcode");
-        steps_ignore.insert("start_filament_gcode");
-        steps_ignore.insert("toolchange_gcode");
-        steps_ignore.insert("threads");
-        steps_ignore.insert("travel_speed");
-        steps_ignore.insert("use_firmware_retraction");
-        steps_ignore.insert("use_relative_e_distances");
-        steps_ignore.insert("use_volumetric_e");
-        steps_ignore.insert("variable_layer_height");
-        steps_ignore.insert("wipe");
-    }
+    static std::unordered_set<std::string> steps_ignore = {
+        "avoid_crossing_perimeters",
+        "bed_shape",
+        "bed_temperature",
+        "before_layer_gcode",
+        "between_objects_gcode",
+        "bridge_acceleration",
+        "bridge_fan_speed",
+        "cooling",
+        "default_acceleration",
+        "deretract_speed",
+        "disable_fan_first_layers",
+        "duplicate_distance",
+        "end_gcode",
+        "end_filament_gcode",
+        "extrusion_axis",
+        "extruder_clearance_height",
+        "extruder_clearance_radius",
+        "extruder_colour",
+        "extruder_offset",
+        "extrusion_multiplier",
+        "fan_always_on",
+        "fan_below_layer_time",
+        "filament_colour",
+        "filament_diameter",
+        "filament_density",
+        "filament_notes",
+        "filament_cost",
+        "filament_max_volumetric_speed",
+        "first_layer_acceleration",
+        "first_layer_bed_temperature",
+        "first_layer_speed",
+        "gcode_comments",
+        "gcode_flavor",
+        "infill_acceleration",
+        "infill_first",
+        "layer_gcode",
+        "min_fan_speed",
+        "max_fan_speed",
+        "min_print_speed",
+        "max_print_speed",
+        "max_volumetric_speed",
+        "max_volumetric_extrusion_rate_slope_positive",
+        "max_volumetric_extrusion_rate_slope_negative",
+        "notes",
+        "only_retract_when_crossing_perimeters",
+        "output_filename_format",
+        "perimeter_acceleration",
+        "post_process",
+        "printer_notes",
+        "retract_before_travel",
+        "retract_before_wipe",
+        "retract_layer_change",
+        "retract_length",
+        "retract_length_toolchange",
+        "retract_lift",
+        "retract_lift_above",
+        "retract_lift_below",
+        "retract_restart_extra",
+        "retract_restart_extra_toolchange",
+        "retract_speed",
+        "slowdown_below_layer_time",
+        "standby_temperature_delta",
+        "start_gcode",
+        "start_filament_gcode",
+        "toolchange_gcode",
+        "threads",
+        "travel_speed",
+        "use_firmware_retraction",
+        "use_relative_e_distances",
+        "use_volumetric_e",
+        "variable_layer_height",
+        "wipe"
+    };
 
     std::vector<PrintStep> steps;
     std::vector<PrintObjectStep> osteps;
@@ -567,8 +567,8 @@ std::string Print::validate() const
             if (std::abs(dmr - 0.4) > EPSILON)
                 return "The Wipe Tower is currently only supported for the 0.4mm nozzle diameter.";
         #endif
-        if (this->config.gcode_flavor != gcfRepRap)
-            return "The Wipe Tower is currently only supported for the RepRap (Marlin / Sprinter) G-code flavor.";
+        if (this->config.gcode_flavor != gcfRepRap && this->config.gcode_flavor != gcfMarlin)
+            return "The Wipe Tower is currently only supported for the Marlin and RepRap/Sprinter G-code flavors.";
         if (! this->config.use_relative_e_distances)
             return "The Wipe Tower is currently only supported with the relative extruder addressing (use_relative_e_distances=1).";
         SlicingParameters slicing_params0 = this->objects.front()->slicing_parameters();
@@ -707,7 +707,10 @@ double Print::skirt_first_layer_height() const
 Flow Print::brim_flow() const
 {
     ConfigOptionFloatOrPercent width = this->config.first_layer_extrusion_width;
-    if (width.value == 0) width = this->regions.front()->config.perimeter_extrusion_width;
+    if (width.value == 0) 
+        width = this->regions.front()->config.perimeter_extrusion_width;
+    if (width.value == 0) 
+        width = this->objects.front()->config.extrusion_width;
     
     /* We currently use a random region's perimeter extruder.
        While this works for most cases, we should probably consider all of the perimeter
@@ -726,7 +729,10 @@ Flow Print::brim_flow() const
 Flow Print::skirt_flow() const
 {
     ConfigOptionFloatOrPercent width = this->config.first_layer_extrusion_width;
-    if (width.value == 0) width = this->regions.front()->config.perimeter_extrusion_width;
+    if (width.value == 0) 
+        width = this->regions.front()->config.perimeter_extrusion_width;
+    if (width.value == 0)
+        width = this->objects.front()->config.extrusion_width;
     
     /* We currently use a random object's support material extruder.
        While this works for most cases, we should probably consider all of the support material
@@ -968,16 +974,53 @@ void Print::_make_wipe_tower()
 
     // Let the ToolOrdering class know there will be initial priming extrusions at the start of the print.
     m_tool_ordering = ToolOrdering(*this, (unsigned int)-1, true);
-    unsigned int initial_extruder_id = m_tool_ordering.first_extruder();
     if (! m_tool_ordering.has_wipe_tower())
         // Don't generate any wipe tower.
         return;
 
+    // Check whether there are any layers in m_tool_ordering, which are marked with has_wipe_tower,
+    // they print neither object, nor support. These layers are above the raft and below the object, and they
+    // shall be added to the support layers to be printed.
+    // see https://github.com/prusa3d/Slic3r/issues/607
+    {
+        size_t idx_begin = size_t(-1);
+        size_t idx_end   = m_tool_ordering.layer_tools().size();
+        // Find the first wipe tower layer, which does not have a counterpart in an object or a support layer.
+        for (size_t i = 0; i < idx_end; ++ i) {
+            const ToolOrdering::LayerTools &lt = m_tool_ordering.layer_tools()[i];
+            if (lt.has_wipe_tower && ! lt.has_object && ! lt.has_support) {
+                idx_begin = i;
+                break;
+            }
+        }
+        if (idx_begin != size_t(-1)) {
+            // Find the position in this->objects.first()->support_layers to insert these new support layers.
+            double wipe_tower_new_layer_print_z_first = m_tool_ordering.layer_tools()[idx_begin].print_z;
+            SupportLayerPtrs::iterator it_layer = this->objects.front()->support_layers.begin();
+            SupportLayerPtrs::iterator it_end   = this->objects.front()->support_layers.end();
+            for (; it_layer != it_end && (*it_layer)->print_z - EPSILON < wipe_tower_new_layer_print_z_first; ++ it_layer);
+            // Find the stopper of the sequence of wipe tower layers, which do not have a counterpart in an object or a support layer.
+            for (size_t i = idx_begin; i < idx_end; ++ i) {
+                ToolOrdering::LayerTools &lt = const_cast<ToolOrdering::LayerTools&>(m_tool_ordering.layer_tools()[i]);
+                if (! (lt.has_wipe_tower && ! lt.has_object && ! lt.has_support))
+                    break;
+                lt.has_support = true;
+                // Insert the new support layer.
+                double height    = lt.print_z - m_tool_ordering.layer_tools()[i-1].print_z;
+                //FIXME the support layer ID is set to -1, as Vojtech hopes it is not being used anyway.
+                auto  *new_layer = new SupportLayer(size_t(-1), this->objects.front(),
+                    height, lt.print_z, lt.print_z - 0.5 * height);
+                it_layer = this->objects.front()->support_layers.insert(it_layer, new_layer);
+                ++ it_layer;
+            }
+        }
+    }
+
     // Initialize the wipe tower.
     WipeTowerPrusaMM wipe_tower(
         float(this->config.wipe_tower_x.value),     float(this->config.wipe_tower_y.value), 
         float(this->config.wipe_tower_width.value), float(this->config.wipe_tower_per_color_wipe.value),
-        initial_extruder_id);
+        m_tool_ordering.first_extruder());
     
     //wipe_tower.set_retract();
     //wipe_tower.set_zhop();
@@ -1000,7 +1043,8 @@ void Print::_make_wipe_tower()
 
     // Generate the wipe tower layers.
     m_wipe_tower_tool_changes.reserve(m_tool_ordering.layer_tools().size());
-    unsigned int current_extruder_id = initial_extruder_id;
+    // Set current_extruder_id to the last extruder primed.
+    unsigned int current_extruder_id = m_tool_ordering.all_extruders().back();
     for (const ToolOrdering::LayerTools &layer_tools : m_tool_ordering.layer_tools()) {
         if (! layer_tools.has_wipe_tower)
             // This is a support only layer, or the wipe tower does not reach to this height.
@@ -1015,7 +1059,11 @@ void Print::_make_wipe_tower()
             last_layer);
         std::vector<WipeTower::ToolChangeResult> tool_changes;
         for (unsigned int extruder_id : layer_tools.extruders)
-            if ((first_layer && extruder_id == initial_extruder_id) || extruder_id != current_extruder_id) {
+            // Call the wipe_tower.tool_change() at the first layer for the initial extruder 
+            // to extrude the wipe tower brim,
+            if ((first_layer && extruder_id == m_tool_ordering.all_extruders().back()) || 
+            // or when an extruder shall be switched.
+                extruder_id != current_extruder_id) {
                 tool_changes.emplace_back(wipe_tower.tool_change(extruder_id, extruder_id == layer_tools.extruders.back(), WipeTower::PURPOSE_EXTRUDE));
                 current_extruder_id = extruder_id;
             }
@@ -1065,7 +1113,11 @@ void Print::_make_wipe_tower()
 std::string Print::output_filename()
 {
     this->placeholder_parser.update_timestamp();
-    return this->placeholder_parser.process(this->config.output_filename_format.value, 0);
+    try {
+        return this->placeholder_parser.process(this->config.output_filename_format.value, 0);
+    } catch (std::runtime_error &err) {
+        throw std::runtime_error(std::string("Failed processing of the output_filename_format template.\n") + err.what());
+    }
 }
 
 std::string Print::output_filepath(const std::string &path)
@@ -1079,13 +1131,13 @@ std::string Print::output_filepath(const std::string &path)
             if (! input_file.empty())
                 break;
         }
-        return (boost::filesystem::path(input_file).parent_path() / this->output_filename()).string();
+        return (boost::filesystem::path(input_file).parent_path() / this->output_filename()).make_preferred().string();
     }
     
     // if we were supplied a directory, use it and append our automatically generated filename
     boost::filesystem::path p(path);
     if (boost::filesystem::is_directory(p))
-        return (p / this->output_filename()).string();
+        return (p / this->output_filename()).make_preferred().string();
     
     // if we were supplied a file which is not a directory, use it
     return path;
