@@ -1,5 +1,6 @@
 #include <limits>
 
+#include "libslic3r.h"
 #include "Slicing.hpp"
 #include "SlicingAdaptive.hpp"
 #include "PrintConfig.hpp"
@@ -21,7 +22,7 @@ namespace Slic3r
 {
 
 static const coordf_t MIN_LAYER_HEIGHT = 0.01;
-static const coordf_t MIN_LAYER_HEIGHT_DEFAULT = 0.05;
+static const coordf_t MIN_LAYER_HEIGHT_DEFAULT = 0.07;
 
 // Minimum layer height for the variable layer height algorithm.
 inline coordf_t min_layer_height_from_nozzle(const PrintConfig &print_config, int idx_nozzle)
@@ -308,19 +309,6 @@ std::vector<coordf_t> layer_height_profile_adaptive(
     return layer_height_profile;
 }
 
-template <typename T>
-static inline T clamp(const T low, const T high, const T value)
-{
-    return std::max(low, std::min(high, value));
-}
-
-template <typename T>
-static inline T lerp(const T a, const T b, const T t)
-{
-    assert(t >= T(-EPSILON) && t <= T(1.+EPSILON));
-    return (1. - t) * a + t * b;
-}
-
 void adjust_layer_height_profile(
     const SlicingParameters     &slicing_params,
     std::vector<coordf_t> 		&layer_height_profile,
