@@ -197,10 +197,11 @@ TriangleMesh::repair() {
         stl_fill_holes(&stl);
         stl_clear_error(&stl);
     }
-    
-    // normal_directions
-    stl_fix_normal_directions(&stl);
-    
+
+    // commenting out the following call fixes: #574, #413, #269, #262, #259, #230, #228, #206
+//    // normal_directions
+//    stl_fix_normal_directions(&stl);
+
     // normal_values
     stl_fix_normal_values(&stl);
     
@@ -1208,6 +1209,8 @@ void TriangleMeshSlicer::make_expolygons(const Polygons &loops, ExPolygons* slic
 
     // perform a safety offset to merge very close facets (TODO: find test case for this)
     double safety_offset = scale_(0.0499);
+//FIXME see https://github.com/prusa3d/Slic3r/issues/520
+//    double safety_offset = scale_(0.0001);
     ExPolygons ex_slices = offset2_ex(p_slices, +safety_offset, -safety_offset);
     
     #ifdef SLIC3R_TRIANGLEMESH_DEBUG
