@@ -158,6 +158,7 @@ public:
     BoundingBoxf3 bounding_box() const {
         BoundingBoxf3 bbox;
         if (! this->vertices_and_normals_interleaved.empty()) {
+            bbox.defined = true;
             bbox.min.x = bbox.max.x = this->vertices_and_normals_interleaved[3];
             bbox.min.y = bbox.max.y = this->vertices_and_normals_interleaved[4];
             bbox.min.z = bbox.max.z = this->vertices_and_normals_interleaved[5];
@@ -207,8 +208,8 @@ public:
         drag_group_id(-1),
         selected(false),
         hover(false),
-        qverts_range(0, size_t(-1)),
-        tverts_range(0, size_t(-1))
+        tverts_range(0, size_t(-1)),
+        qverts_range(0, size_t(-1))
     {
         color[0] = r;
         color[1] = g;
@@ -344,14 +345,23 @@ public:
     static void _glew_init();
 
     static void _load_print_toolpaths(
-        const Print         *print,
-        GLVolumeCollection  *volumes,
-        bool                 use_VBOs);
+        const Print                     *print,
+        GLVolumeCollection              *volumes,
+        const std::vector<std::string>  &tool_colors,
+        bool                             use_VBOs);
 
     static void _load_print_object_toolpaths(
-        const PrintObject   *print_object,
-        GLVolumeCollection  *volumes,
-        bool                 use_VBOs);
+        const PrintObject               *print_object,
+        GLVolumeCollection              *volumes,
+        const std::vector<std::string>  &tool_colors,
+        bool                             use_VBOs);
+
+
+    static void _load_wipe_tower_toolpaths(
+        const Print                    *print,
+        GLVolumeCollection             *volumes,
+        const std::vector<std::string> &tool_colors_str,
+        bool                            use_VBOs);
 };
 
 }
